/* A function for counting the sanbox's particles.
   Copyright (C) 2006 Carl Fredrik Hammar (hammy.lite@gmail.com)

   This file is part of GP2X Sand.

   GP2X Sand is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 2 of the License, or (at your
   option) any later version.

   GP2X Sand is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
   General Public License for more details. 

   You should have received a copy of the GNU General Public License along
   with GP2X Sand; if not, write to the Free Software Foundation, Inc., 51
   Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA */

#include "sandbox.h"

/* Counts the number of particles in the sandbox and returns the result. */
unsigned int
sandbox_particles_count ()
{
  sandbox_sector_t *sectors;
  int sectors_length;
  int particle_count = 0;
  
  sandbox_sectors_get (&sectors, &sectors_length);
  
  for (int i = 0; i < sectors_length; i++)
    {
      if (sectors[i].homogenous_flag && !sectors[i].dirty_flag)
	{
	  element_t *dominant_element_ptr
	    = sectors[i].dominant_element_ptr;

	  if (dominant_element_ptr != element_nothing_ptr)
	    particle_count += sectors[i].dimensions.x
	      * sectors[i].dimensions.y;
	}
      else
	{
	  /* Position of the sector's top-left corner. */ 
	  vect_t start = { sectors[i].position.x,
			   sectors[i].position.y };

	  /* Position of the sector's bottom-right corner. */ 
	  vect_t end = { start.x + sectors[i].dimensions.x,
			 start.y + sectors[i].dimensions.y };
      
	  for (vect_t v = start; v.y < end.y; v.y++)
	    {
	      for (v.x = start.x; v.x < end.x; v.x++)
		{
		  particle_t particle = sandbox_particle_get (v);
		  element_t *element_ptr =
		    particle_element_get (particle);
	      
		  if (element_ptr != element_nothing_ptr)
		    particle_count++;
		}
	    }
	}
    }

  return particle_count;
}
