/* Small and fast functions for working with probabilities.
   Copyright (C) 2006 Carl Fredrik Hammar (hammy.lite@gmail.com)

   This file is part of GP2X Sand.

   GP2X Sand is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by the
   Free Software Foundation; either version 2 of the License, or (at your
   option) any later version.

   GP2X Sand is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
   General Public License for more details. 

   You should have received a copy of the GNU General Public License along
   with GP2X Sand; if not, write to the Free Software Foundation, Inc., 51
   Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA */

#ifndef PROBABILITY_H
#define PROBABILITY_H

#include <assert.h>
#include <stdbool.h>
#include <stdint.h>
#include <stdlib.h>

#ifndef INLINE
#define INLINE extern inline
#endif

/* Probabilities are stored as parts of 2^PROB_SHIFT.
   Assuming RAND_MAX equals INT_MAX. */
#define PROB_SHIFT (31)
#define PROB_MAX (RAND_MAX) /* Assuming RAND_MAX equals INT_MAX. */

typedef uint32_t prob_t;

/* Return a probability value equaling to NUMERATOR / DENOMINATOR.
   NUMERATOR must be less then or equal to DENOMINATOR. */
INLINE
prob_t
prob_cons (uint32_t numerator, uint32_t denominator)
{
  assert (numerator <= denominator);
  
  if (numerator == 0)
    return 0;
  else
    return ((uint64_t) numerator * PROB_MAX / denominator);
}

/* Return the probability of both P1 and P2 happening. */ 
INLINE
prob_t
prob_compose (prob_t p1, prob_t p2)
{
  return ((uint64_t) p1 * p2) >> PROB_SHIFT;
}

/* Return true if event with probability of PROBABILITY should happen
   now.  For use in if statements preceding the events code. */
INLINE
bool
prob_boolean (prob_t probability)
{
  return probability == PROB_MAX || rand () < probability;
}

#endif /* PROBABILITY_H */
